-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

local thModName = g_currentModName
local thModPath = g_currentModDirectory
source(thModPath .. "scripts/utilities/THUtils.lua")
THDefaultTypes = {}
local THDefaultTypes_mt = THUtils.createClass(THDefaultTypes)
function THDefaultTypes.new(customMt)
    customMt = customMt or THDefaultTypes_mt
    if THUtils.argIsValid(type(customMt) == "table", "customMt", customMt) then
        local self = setmetatable({}, customMt)
        self.isServer = g_server ~= nil
        self.isClient = g_client ~= nil
        self.modName = thModName
        self.modPath = thModPath
        self.fruitTypeManager = _G.g_fruitTypeManager
        self.defaults = {}
        return self
    end
end
function THDefaultTypes.loadDefaultTypeData(self, missionInfo, xmlFile, dataKey, baseDirectory)
    if THUtils.argIsValid(type(missionInfo) == "table", "missionInfo", missionInfo)
        and THUtils.argIsValid(type(dataKey) == "string", "dataKey", dataKey)
        and THUtils.argIsValid(baseDirectory == nil or type(baseDirectory) == "string", "baseDirectory", baseDirectory)
    then
        local mapDesc = missionInfo.map
        if mapDesc ~= nil then
            local entryKey = string.format("map.%s", dataKey)
            local entryFilename = THUtils.getXMLValue(xmlFile, XMLValueType.STRING, entryKey .. "#filename")
            local isDefaultTypes = THUtils.getXMLValue(xmlFile, XMLValueType.BOOL, entryKey .. "#thDefaultTypes", false)
            if entryFilename ~= nil then
                entryFilename = THUtils.getFilename(entryFilename, baseDirectory)
                if isDefaultTypes and entryFilename ~= nil and fileExists(entryFilename) then
                    local typeData = {
                        xmlFilename = entryFilename,
                        mapXMLFile = xmlFile,
                        modPath = baseDirectory,
                        modName = mapDesc.customEnvironment,
                        isModMap = mapDesc.isModMap,
                        isMapLoading = false
                    }
                    return typeData
                end
            end
        end
    end
end
function THDefaultTypes.inj_loadDataFromMapXML(self, superFunc, mapXMLFile, xmlKey, ...)
    local typeData = nil
    if mapXMLFile ~= nil and xmlKey ~= nil then
        THUtils.call(function()
            typeData = self.defaults[xmlKey]
            if typeData ~= nil and mapXMLFile == typeData.mapXMLFile then
                typeData.isMapLoading = true
            end
        end)
    end
    local function appendFunc(rSuccess, ...)
        if typeData ~= nil and typeData.isMapLoading then
            typeData.isMapLoading = false
        end
        return rSuccess, ...
    end
    return appendFunc(superFunc(mapXMLFile, xmlKey, ...))
end
function THDefaultTypes.inj_loadFillTypeMapData(self, superFunc, manager, xmlFile, missionInfo, baseDirectory, ...)
    if xmlFile ~= nil and missionInfo ~= nil then
        THUtils.call(function()
            if self.defaults.fillTypes == nil then
                self.defaults.fillTypes = self:loadDefaultTypeData(missionInfo, xmlFile, "fillTypes", baseDirectory)
            end
        end)
    end
    return superFunc(manager, xmlFile, missionInfo, baseDirectory, ...)
end
function THDefaultTypes.inj_loadFillTypes(self, superFunc, manager, xmlFile, baseDirectory, isBaseType, modName, ...)
    local skipLoad = false
    THUtils.call(function()
        local typeData = self.defaults.fillTypes
        if typeData ~= nil then
            if isBaseType == true then
                skipLoad = true
                THUtils.displayMsg("Skipping vanilla fillType registration...")
            elseif typeData.isMapLoading then
                isBaseType = true
                THUtils.displayMsg("Loading fillType defaults from: %s", typeData.xmlFilename)
            end
        end
    end)
    if skipLoad then
        return true
    end
    return superFunc(manager, xmlFile, baseDirectory, isBaseType, modName, ...)
end
function THDefaultTypes.inj_loadFruitTypeMapData(self, superFunc, manager, xmlFile, missionInfo, baseDirectory, ...)
    if xmlFile ~= nil and missionInfo ~= nil then
        THUtils.call(function()
            if self.defaults.fruitTypes == nil then
                self.defaults.fruitTypes = self:loadDefaultTypeData(missionInfo, xmlFile, "fruitTypes", baseDirectory)
            end
        end)
    end
    return superFunc(manager, xmlFile, missionInfo, baseDirectory, ...)
end
function THDefaultTypes.inj_loadFruitTypes(self, superFunc, manager, xmlFile, missionInfo, baseDirectory, isBaseType, ...)
    local skipLoad = false
    THUtils.call(function()
        local typeData = self.defaults.fruitTypes
        if typeData ~= nil then
            if isBaseType == true then
                skipLoad = true
                THUtils.displayMsg("Skipping vanilla fruitType registration...")
            elseif typeData.isMapLoading then
                isBaseType = true
                THUtils.displayMsg("Loading fruitType defaults from: %s", typeData.xmlFilename)
            end
        end
    end)
    if skipLoad then
        return true
    end
    return superFunc(manager, xmlFile, missionInfo, baseDirectory, isBaseType, ...)
end
function THDefaultTypes.inj_loadFoliageFromXMLFile(self, superFunc, desc, filename, ...)
    local isFruitTypeSwapped = false
    if desc ~= nil and filename ~= nil then
        THUtils.call(function()
            if fileExists(filename) then
                local fruitXMLFile = THUtils.loadXMLFile("THFruitType", filename, nil, false)
                if THUtils.getNoNil(fruitXMLFile, 0) > 0 then
                    local fruitTypeName = getXMLString(fruitXMLFile, "foliageType.fruitType#name")
                    if fruitTypeName ~= nil then
                        fruitTypeName = fruitTypeName:upper()
                        local otherDesc = self.fruitTypeManager:getFruitTypeByName(fruitTypeName)
                        if otherDesc ~= nil and otherDesc.index ~= desc.index then
                            THUtils.errorMsg(false, "FruitType %q already registered at index %s, updating existing entry...", fruitTypeName, otherDesc.index)
                            desc = otherDesc
                            isFruitTypeSwapped = true
                        end
                    end
                    THUtils.deleteXMLFile(fruitXMLFile)
                end
            end
        end)
    end
    local function appendFunc(rSuccess, ...)
        if rSuccess then
            THUtils.call(function()
                if isFruitTypeSwapped then
                    rSuccess = false
                end
            end)
        end
        return rSuccess, ...
    end
    return appendFunc(superFunc(desc, filename, ...))
end
THUtils.call(function()
    local thMain = THDefaultTypes.new()
    if thMain ~= nil then
        _G.g_thDefaultTypes = thMain
        THUtils.setFunctionHook("XMLUtil", "loadDataFromMapXML", false, false, thMain, THDefaultTypes.inj_loadDataFromMapXML)
        THUtils.setFunctionHook("FillTypeManager", "loadMapData", false, false, thMain, THDefaultTypes.inj_loadFillTypeMapData)
        THUtils.setFunctionHook("FillTypeManager", "loadFillTypes", false, false, thMain, THDefaultTypes.inj_loadFillTypes)
        THUtils.setFunctionHook("FruitTypeManager", "loadMapData", false, false, thMain, THDefaultTypes.inj_loadFruitTypeMapData)
        THUtils.setFunctionHook("FruitTypeManager", "loadFruitTypes", false, false, thMain, THDefaultTypes.inj_loadFruitTypes)
        THUtils.setFunctionHook("FruitTypeDesc", "loadFromFoliageXMLFile", false, false, thMain, THDefaultTypes.inj_loadFoliageFromXMLFile)
    end
end)